"""
Test models used in django-klingon tests
"""
from django.db import models
from klingon.models import Translatable, AutomaticTranslation

from autoslug import AutoSlugField


class Book(models.Model, Translatable):
    title = models.CharField(max_length=100)
    description = models.TextField()
    publication_date = models.DateField()
    slug = models.SlugField()

    translatable_fields = ('title', 'description')
    translatable_slug = 'slug'

    def __unicode__(self):
        return self.title


class Library(AutomaticTranslation, models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField()
    slug = AutoSlugField(populate_from='name')

    translatable_fields = ('name', 'description')
    translatable_slug = 'slug'

    def __unicode__(self):
        return self.name



from django import forms
from django.contrib import admin
from django.forms.models import ModelForm
from django.utils.translation import ugettext as _
from .models import Translation
from .compat import GenericTabularInline


class TranslationAdmin(admin.ModelAdmin):
    list_display = ('content_object', 'lang', 'field')
    search_fields = ('lang', 'field', 'translation')
    list_filter = ('lang', 'field', 'content_type')


admin.site.register(Translation, TranslationAdmin)


def create_translations(modeladmin, request, queryset):
    for obj in queryset:
        obj.translate()


create_translations.short_description = _('Create translations for selected objects')


class TranslationInlineForm(ModelForm):
    widgets = {
        'CharField': forms.TextInput(),
        'TextField': forms.Textarea(),
    }

    class Meta:
        model = Translation
        exclude = []

    def __init__(self, *args, **kwargs):
        res = super(TranslationInlineForm, self).__init__(*args, **kwargs)
        # overwrite the widgets for each form instance depending on the object and widget dict

        if self.widgets and self.instance and hasattr(self.instance, 'content_type'):
            model = self.instance.content_type.model_class()
            field = model._meta.get_field(self.instance.field)
            # get widget for field if any - by name or by type
            widget = self.widgets.get(field.name) or \
                self.widgets.get(field.get_internal_type())

            if widget:
                translation = self.fields['translation']
                # overwrite widget to field
                translation.widget = widget

        return res

    def clean_translation(self):
        """
        Do not allow translations longer than the max_lenght of the field to
        be translated.
        """
        translation = self.cleaned_data['translation']

        if self.instance and self.instance.content_object:
            # do not allow string longer than translatable field
            obj = self.instance.content_object
            field = obj._meta.get_field(self.instance.field)
            max_length = field.max_length

            if max_length and len(translation) > max_length:
                raise forms.ValidationError(
                    _('The entered translation is too long. You entered '
                      '%(entered)s chars, max length is %(maxlength)s') % {
                        'entered': len(translation),
                        'maxlength': max_length,
                    }
                )
        else:
            raise forms.ValidationError(
                _('Can not store translation. First create all translation'
                  ' for this object')
            )
        return translation


class TranslationInline(GenericTabularInline):
    model = Translation
    form = TranslationInlineForm

    extra = 0
    readonly_fields = ('lang', 'field')
    exclude = ('lang', 'field')
    can_delete = False



# flake8: ignore=F401
from django.conf import settings
try:
    from django.core import urlresolvers
except (ModuleNotFoundError, ImportError):
    from django import urls as urlresolvers

from django.core.cache import cache
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext as _

INSTALLED_AUTOSLUG = False

try:
    from autoslug import AutoSlugField
    from autoslug.settings import slugify

    INSTALLED_AUTOSLUG = True
except ImportError:
    pass

from klingon.compat import GenericForeignKey


class Translation(models.Model):
    """
    Model that stores all translations
    """
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()
    lang = models.CharField(max_length=5, db_index=True)
    field = models.CharField(max_length=255, db_index=True)
    translation = models.TextField(blank=True, null=True)

    class Meta:
        ordering = ['lang', 'field']
        unique_together = (('content_type', 'object_id', 'lang', 'field'),)

    def __unicode__(self):
        return u'%s : %s : %s' % (self.content_object, self.lang, self.field)


class CanNotTranslate(Exception):
    pass


class Translatable(object):
    """
    Make your model extend this class to enable this API in you model instance
    instances.
    """
    translatable_fields = ()
    translatable_slug = None

    def translate(self):
        """
        Create all translations objects for this Translatable instance.

        @rtype: list of Translation objects
        @return: Returns a list of translations objects
        """
        translations = []
        for lang in settings.LANGUAGES:
            # do not create an translations for default language.
            # we will use the original model for this
            if lang[0] == self._get_default_language():
                continue
            # create translations for all fields of each language
            if self.translatable_slug is not None:
                if self.translatable_slug not in self.translatable_fields:
                    self.translatable_fields = self.translatable_fields + (self.translatable_slug,)

            for field in self.translatable_fields:
                trans, created = Translation.objects.get_or_create(
                    object_id=self.id,
                    content_type=ContentType.objects.get_for_model(self),
                    field=field,
                    lang=lang[0],
                )
                translations.append(trans)

        return translations

    def translations_objects(self, lang):
        """
        Return the complete list of translation objects of a Translatable
        instance

        @type lang: string
        @param lang: a string with the name of the language

        @rtype: list of Translation
        @return: Returns a list of translations objects
        """
        return Translation.objects.filter(
            object_id=self.id,
            content_type=ContentType.objects.get_for_model(self),
            lang=lang
        )

    def translations(self, lang):
        """
        Return the list of translation strings of a Translatable
        instance in a dictionary form

        @type lang: string
        @param lang: a string with the name of the language

        @rtype: python Dictionary
        @return: Returns a all fieldname / translations (key / value)
        """
        key = self._get_translations_cache_key(lang)
        trans_dict = cache.get(key, {})

        if self.translatable_slug is not None:
            if self.translatable_slug not in self.translatable_fields:
                self.translatable_fields = self.translatable_fields + (self.translatable_slug,)

        if not trans_dict:
            for field in self.translatable_fields:
                # we use get_translation method to be sure that it will
                # fall back and get the default value if needed
                trans_dict[field] = self.get_translation(lang, field)

            cache.set(key, trans_dict)
        return trans_dict

    def get_translation_obj(self, lang, field, create=False):
        """
        Return the translation object of an specific field in a Translatable
        istance

        @type lang: string
        @param lang: a string with the name of the language

        @type field: string
        @param field: a string with the name that we try to get

        @rtype: Translation
        @return: Returns a translation object
        """
        trans = None
        try:
            trans = Translation.objects.get(
                object_id=self.id,
                content_type=ContentType.objects.get_for_model(self),
                lang=lang,
                field=field,
            )
        except Translation.DoesNotExist:
            if create:
                trans = Translation.objects.create(
                    object_id=self.id,
                    content_type=ContentType.objects.get_for_model(self),
                    lang=lang,
                    field=field,
                )
        return trans

    def get_translation(self, lang, field):
        """
        Return the translation string of an specific field in a Translatable
        istance

        @type lang: string
        @param lang: a string with the name of the language

        @type field: string
        @param field: a string with the name that we try to get

        @rtype: string
        @return: Returns a translation string
        """
        # Read from cache
        key = self._get_translation_cache_key(lang, field)
        trans = cache.get(key, '')

        if not trans:
            trans_obj = self.get_translation_obj(lang, field)
            trans = getattr(trans_obj, 'translation', '')
            # if there's no translation text fall back to the model field
            if not trans:
                trans = getattr(self, field, '')
            # update cache
            cache.set(key, trans)
        return trans

    def set_translation(self, lang, field, text):
        """
        Store a translation string in the specified field for a Translatable
        istance

        @type lang: string
        @param lang: a string with the name of the language

        @type field: string
        @param field: a string with the name that we try to get

        @type text: string
        @param text: a string to be stored as translation of the field
        """
        # Do not allow user to set a translations in the default language
        auto_slug_obj = None

        if lang == self._get_default_language():
            raise CanNotTranslate(
                _('You are not supposed to translate the default language. '
                  'Use the model fields for translations in default language')
            )

        # Get translation, if it does not exits create one
        trans_obj = self.get_translation_obj(lang, field, create=True)
        trans_obj.translation = text
        trans_obj.save()

        # check if the field has an autoslugfield and create the translation
        if INSTALLED_AUTOSLUG:
            if self.translatable_slug:
                try:
                    auto_slug_obj = self._meta.get_field(self.translatable_slug).populate_from
                except AttributeError:
                    pass

        if auto_slug_obj:
            tobj = self.get_translation_obj(lang, self.translatable_slug, create=True)
            translation = self.get_translation(lang, auto_slug_obj)
            tobj.translation = slugify(translation)
            tobj.save()

        # Update cache for this specif translations
        key = self._get_translation_cache_key(lang, field)
        cache.set(key, text)
        # remove cache for translations dict
        cache.delete(self._get_translations_cache_key(lang))
        return trans_obj

    def translations_link(self):
        """
        Print on admin change list the link to see all translations for this object

        @type text: string
        @param text: a string with the html to link to the translations admin interface
        """
        translation_type = ContentType.objects.get_for_model(Translation)
        link = urlresolvers.reverse('admin:%s_%s_changelist' % (
            translation_type.app_label,
            translation_type.model),
        )

        object_type = ContentType.objects.get_for_model(self)
        link += '?content_type__id__exact=%s&object_id=%s' % (object_type.id, self.id)
        return '<a href="%s">translate</a>' % link

    translations_link.allow_tags = True
    translations_link.short_description = 'Translations'

    def _get_default_language(self):
        """
        Helper function to get defaul setting for klingon.
        This is not set at module level to make it easy to test
        """
        return getattr(settings, 'KLINGON_DEFAULT_LANGUAGE', '')

    def _get_translations_cache_key(self, lang):
        content_type = self._meta.object_name
        return '%s:%s:%s' % (content_type, self.id, lang)

    def _get_translation_cache_key(self, lang, field):
        content_type = self._meta.object_name
        return '%s:%s:%s:%s' % (content_type, self.id, lang, field)


class AutomaticTranslation(Translatable):
    """
    Model that automatically crates translations
    """

    def save(self, *args, **kwargs):
        res = super(AutomaticTranslation, self).save(*args, **kwargs)
        self.translate()
        return res



def pytest_configure():
    from django.conf import settings
    if not settings.configured:
        settings.configure(
            DEBUG_PROPAGATE_EXCEPTIONS=True,
            DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3',
                       'NAME': ':memory:'}},
            INSTALLED_APPS=(
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'django.contrib.admin',
                'autoslug',
                'klingon',
                'tests',
                'tests.testapp',
            ),
            SITE_ID=1,
            SECRET_KEY='this-is-just-for-tests-so-not-that-secret',
            LANGUAGES = (
                ('en', 'English'),
                ('pt_br', 'Brazilian Portuguese'),
                ('es', 'Spanish'),
            ),
            MIDDLEWARE_CLASSES=(),
        )

        try:
            import django
            django.setup()
        except AttributeError:
            pass



import datetime

from django.conf import settings
from django.core.cache import cache
from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase
from django.utils.text import slugify
from klingon.models import Translation

from .testapp.models import Book


class CommandsTestCase(TestCase):
    def setUp(self):
        self.book = Book.objects.create(
            title="The Raven",
            description="The Raven is a narrative poem",
            publication_date=datetime.date(1845, 1, 1),
            slug="the-raven",
        )
        self.es_title = u"El Cuervo"
        self.es_description = 'El Cuervo es un poema narrativo'
        self.es_slug = slugify(self.es_title)

    def tearDown(self):
        # Remove cache after each test
        cache.clear()

    def _test_translate_models_command(self, args):
        args = args
        opts = {}
        call_command('translatemodels', *args, **opts)

    def test_translate_models_command_success(self):
        self._test_translate_models_command(['testapp.Book'])

        ifslug = 0
        if self.book.translatable_slug:
            ifslug = 1

        self.assertEqual(
            Translation.objects.count(),
            len(settings.LANGUAGES * (len(self.book.translatable_fields) + ifslug)),
        )

    def test_translate_models_command_empty(self):
        self._test_translate_models_command([])
        self.assertEqual(Translation.objects.count(), 0)

    def test_translate_models_command_wrong_args(self):
        self.assertRaises(
            CommandError,
            self._test_translate_models_command,
            ['wrong.argument']
        )



from __future__ import (division, print_function, unicode_literals,
                        with_statement)

from django import VERSION
from django.apps.registry import apps
from django.contrib.contenttypes.admin import GenericTabularInline

from django.contrib.contenttypes.fields import GenericForeignKey

version = VERSION[1]

V17 = 7
V18 = 8
V19 = 9
V110 = 10

get_model = apps.get_model

__all__ = ['get_model', 'GenericForeignKey', 'GenericTabularInline']



import datetime
from django.conf import settings
from django.core.cache import cache
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.text import slugify
from klingon.models import CanNotTranslate, Translation
from .testapp.models import Book, Library


class TranslationAPITestCase(TestCase):
    def setUp(self):
        self.book = Book.objects.create(
            title="The Raven",
            description="The Raven is a narrative poem",
            publication_date=datetime.date(1845, 1, 1),
            slug="the-raven"
        )
        self.es_title = u"El Cuervo"
        self.es_description = 'El Cuervo es un poema narrativo'
        self.es_slug = slugify(self.es_title)

    def tearDown(self):
        # Remove cache after each test
        cache.clear()

    def test_api_general_usage(self):
        # translate book to spanish
        self.book.set_translation('es', 'title', self.es_title)
        self.book.set_translation('es', 'description', self.es_description)
        self.book.set_translation('es', 'slug', self.es_slug)
        # Get translations
        self.assertEqual(
            self.book.get_translation('es', 'title'),
            self.es_title
        )
        self.assertEqual(
            self.book.get_translation('es', 'description'),
            self.es_description,
        )
        self.assertEqual(
            self.book.get_translation('es', 'slug'),
            self.es_slug,
        )
        self.assertEqual(
            self.book.translations('es'),
            {
                'title': self.es_title,
                'description': self.es_description,
                'slug': self.es_slug,
            }
        )

    def test_api_default_value(self):
        self.assertEqual(
            self.book.get_translation('es', 'title'),
            self.book.title
        )
        self.assertEqual(
            self.book.get_translation('es', 'description'),
            self.book.description
        )
        self.assertEqual(
            self.book.get_translation('es', 'slug'),
            self.book.slug
        )

    def test_create_translations(self):
        self.book.translate()

        self.assertEqual(
            Translation.objects.count(),
            len(settings.LANGUAGES * len(self.book.translatable_fields)),
        )
        # nothing should happen if you run translate tiwce
        self.book.translate()

        self.assertEqual(
            len(Translation.objects.all()),
            len(settings.LANGUAGES * len(self.book.translatable_fields)),
        )

    def test_set_translation(self):
        self.book.set_translation('es', 'title', self.es_title)
        self.assertEqual(
            Translation.objects.get(lang='es', field='title').translation,
            self.es_title
        )

    def test_get_translation(self):
        self.book.set_translation('es', 'title', self.es_title)
        es_title = self.book.get_translation('es', 'title')
        self.assertEqual(
            es_title,
            self.es_title
        )

    def test_translations(self):
        # translate book to spanish
        self.book.set_translation('es', 'title', self.es_title)
        self.book.set_translation('es', 'description', self.es_description)
        self.book.set_translation('es', 'slug', self.es_slug)
        # get all spanish translations for the book
        self.assertEqual(
            self.book.translations('es'),
            {'title': self.es_title, 'description': self.es_description, 'slug': self.es_slug}
        )

    def test_translations_cache(self):
        # translate book to spanish
        self.book.set_translation('es', 'title', self.es_title)
        self.book.set_translation('es', 'description', self.es_description)
        self.book.set_translation('es', 'slug', self.es_slug)
        # get translations
        trans = self.book.translations('es')
        # set a new translation
        es_new_title = 'El Cuervo (*)'
        self.book.set_translation('es', 'title', es_new_title)
        # get new translations
        new_trans = self.book.translations('es')
        self.assertNotEqual(trans, new_trans)
        self.assertEqual(
            new_trans,
            {'title': es_new_title, 'description': self.es_description, 'slug': self.es_slug}
        )

    @override_settings(KLINGON_DEFAULT_LANGUAGE='en')
    def test_create_translations_with_default_language(self):
        self.book.translate()

        self.assertEqual(
            len(Translation.objects.all()),
            (len(settings.LANGUAGES) - 1) * len(self.book.translatable_fields)
        )
        # nothing should happen if you run translate tiwce
        self.book.translate()
        self.assertEqual(
            len(Translation.objects.all()),
            (len(settings.LANGUAGES) - 1) * len(self.book.translatable_fields),
        )

    @override_settings(KLINGON_DEFAULT_LANGUAGE='en')
    def test_get_translation_with_default_language(self):
        es_title = self.book.get_translation('es', 'title')
        # Get should not create empty translations
        self.assertEqual(Translation.objects.count(), 0)
        # Get should fall back to default language
        self.assertEqual(es_title, self.book.title)

    @override_settings(KLINGON_DEFAULT_LANGUAGE='en')
    def test_set_translation_with_default_language(self):
        # Verify that there are no translations
        self.assertEqual(Translation.objects.count(), 0)
        # Create a translation
        self.book.set_translation('es', 'title', self.es_title)
        # Get should work as expected
        self.assertEqual(
            self.es_title,
            self.book.get_translation('es', 'title')
        )
        # Verify that only one translation was created
        self.assertEqual(Translation.objects.count(), 1)

    @override_settings(KLINGON_DEFAULT_LANGUAGE='en')
    def test_set_translation_in_default_language(self):
        # Create a translation for default language
        self.assertRaises(
            CanNotTranslate,
            self.book.set_translation,
            'en', 'title', 'The Raven!'
        )


class AutomaticTranslationAPITestCase(TestCase):
    def setUp(self):
        self.library = Library.objects.create(
            name=u"Indepent",
            description="All in ebooks",
        )
        self.es_name = u"Independiente"
        self.es_description = 'Todo en ebooks'
        self.es_slug = slugify(self.es_name)

    def tearDown(self):
        # Remove cache after each test
        cache.clear()

    def test_translation_created_automatically(self):
        self.assertEqual(
            len(Translation.objects.all()),
            len(settings.LANGUAGES * len(self.library.translatable_fields)),
        )

    def test_translation_slug(self):
        self.library.set_translation('es', 'name', self.es_name)
        self.library.set_translation('es', 'description', self.es_description)
        self.assertEqual(self.library.slug, slugify(self.library.name))
        self.assertEqual(self.library.get_translation('es', 'name'),
                          self.es_name)
        self.assertEqual(self.library.get_translation('es', 'slug'),
                          self.es_slug)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Translation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField()),
                ('lang', models.CharField(max_length=5, db_index=True)),
                ('field', models.CharField(max_length=255, db_index=True)),
                ('translation', models.TextField(null=True, blank=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ['lang', 'field'],
            },
        ),
        migrations.AlterUniqueTogether(
            name='translation',
            unique_together=set([('content_type', 'object_id', 'lang', 'field')]),
        ),
    ]



import os, sys
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


version = __import__('klingon').__version__
readme = read_file('README.rst')
history = read_file('HISTORY.rst').replace('.. :changelog:', '')

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

setup(
    name='django-klingon',
    version=version,
    author='Rafael Capdevielle, Angel Velasquez',
    author_email='angvp@archlinux.org',
    packages=find_packages(),
    include_package_data=True,
    url='http://github.com/angvp/django-klingon',
    license='GPL',
    description="""django-klingon is an attempt to make django model translation suck but with no integrations pain in your app!""",
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
    ],
    long_description=readme + '\n\n' + history,
    test_suite="runtests",
    zip_safe=False,
)



from django.core.management.base import BaseCommand, CommandError
from klingon.compat import get_model, V110, version
import os


class Command(BaseCommand):
    args = '<app.Model1 app.Model2 ...>'
    help = 'Create translation for specific models <app>.<model_name>' \
        ' \n Example: \n $ python manage.py translatemodels testapp.Book'

    def add_arguments(self, parser):
        parser.add_argument('args', nargs='*', default=[])

    def handle(self, *args, **options):
        for model_name in args:
            try:
                app, name = model_name.split('.')
                model = get_model(app, name)
                for obj in model.objects.only('pk'):
                    obj.translate()
            except Exception as e:
                raise CommandError('Error, can not translate model "%s". %s' % (model_name, e))
            else:
                self.stdout.write('All translations created for "%s"' % model_name)
